local realmName = GetCVar("realmName")
local skull = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8.blp:0|t"
local DBVersion = 1
local pvpzone

DeathNote = {}
DeathNote.Defaults = { 
	[realmName] = {},
	autoadd = true,
	noteshare = true,
	notereceive = true,
	version = DBVersion,
}

if not DeathNoteDB then DeathNoteDB = DeathNote.Defaults end

local f = CreateFrame("frame")
f:SetScript("OnEvent", function(self, event, ...)
	if self[event] then return self[event](self, event, ...) end
end)

local ZoneBlacklist = {
	[4] = 11, -- Wintergrasp
}
local function IsBlacklistedZone(zone,continent)
	for k,v in pairs(ZoneBlacklist) do
		if zone == k and continent == v then
			return true
		end
	end
end

local function ZoneCheck()
	local instancetype = select(2,GetInstanceInfo())
	if instancetype == "none" then
		DeathNote:Enable()
		pvpzone = nil
	else
		DeathNote:Disable()
		pvpzone = true
	end
end

local ZoneWatcher = CreateFrame("Frame")
ZoneWatcher:RegisterEvent("ZONE_CHANGED_NEW_AREA")
ZoneWatcher:SetScript("OnEvent", ZoneCheck)

function DeathNote:Enable()
	f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	f:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	f:RegisterEvent("PLAYER_REGEN_ENABLED")
	f:RegisterEvent("CHAT_MSG_ADDON")
	f:RegisterEvent("PLAYER_LOGIN")
end

function DeathNote:Disable()
	f:UnregisterAllEvents()
end

DeathNote:Enable()

local function checksourceflags(flag)
	local HOSTILEUNITFLAG = 0x548
	if bit.band(flag,HOSTILEUNITFLAG) == HOSTILEUNITFLAG then
		return true
	end
end

local lastdamage
function f:PLAYER_REGEN_ENABLED()
	lastdamage = nil
end

function f:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, minievent, guidsource, source, sourceflags, ...)
	if not IsBlacklistedZone() then
		if (minievent == "SWING_DAMAGE" or minievent == "SPELL_DAMAGE" or minievent == "SPELL_PERIODIC_DAMAGE") and checksourceflags(sourceflags) then
			lastdamage = source
		end
		if minievent == "UNIT_DIED" and lastdamage then
			local playername = UnitName("player")
			if select(2,...) == playername then
				local nameexists = false
				local zonename = GetZoneText()
				local note = "Killed "..playername.." in "..zonename
				if DeathNoteDB.autoadd then
					for k,v in pairs(DeathNoteDB[realmName]) do
						if string.lower(v[1]) == string.lower(lastdamage) then
							v[2] = note
							nameexists = true
						end
					end
					if not nameexists then
						print("DeathNote: "..lastdamage.." damaged you before you died. Adding in the list.")
						table.insert(DeathNoteDB[realmName],{lastdamage,note})
					end
				end
				if DeathNoteDB.noteshare and GetNumPartyMembers() > 1 then
					SendAddonMessage("DEATHNOTE",lastdamage.."|"..note,"PARTY")
				end
			end
			lastdamage = nil
		end
	end
end

function f:CHAT_MSG_ADDON(self,prefix,message,channel,sender)
	local nameexists = false
	if DeathNoteDB.notereceive and prefix == "DEATHNOTE" then
		local name,note = string.split("|",message)
		for k,v in pairs(DeathNoteDB[realmName]) do
			if string.lower(v[1]) == string.lower(name) then
				v[2] = note
				nameexists = true
			end
		end
		if not nameexists then
			print("DeathNote: Received message from "..sender)
			table.insert(DeathNoteDB[realmName],{name,note})
		end
	end
end

function f:PLAYER_LOGIN()
	if not DeathNoteDB.version or DeathNoteDB.version ~= DBVersion then
		DeathNoteDB = DeathNote.Defaults
		print("DeathNote: Database reset!")
	end
	if not DeathNoteDB[realmName] then DeathNoteDB[realmName] = {} end
	ZoneCheck()
end

local function OnTooltipSetUnit(tooltip, ...)
	local name = UnitName("mouseover")
	for k,v in pairs(DeathNoteDB[realmName]) do
		if name and string.lower(name) == string.lower(v[1]) then
			GameTooltip:AddSpacerLine(1,1,0,0)
			GameTooltip:AddLine(skull.." "..v[2].." "..skull,1,0,0)
		end
	end
	if origsetunit then
		return origsetunit(tooltip, ...)
	end
end

local origsetunit = GameTooltip:GetScript("OnTooltipSetUnit")
GameTooltip:SetScript("OnTooltipSetUnit", OnTooltipSetUnit)

local function ShowItemRefTooltip()
	ShowUIPanel(ItemRefTooltip)
	if not ItemRefTooltip:IsShown() then ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE") end
	ItemRefTooltip:ClearLines()
	ItemRefTooltip:AddLine("DeathNote: People to kill:")
	for k,v in pairs(DeathNoteDB[realmName]) do
		ItemRefTooltip:AddLine(k.." - "..v[1]..": "..v[2])
	end
	ItemRefTooltip:Show()
end

local frame = CreateFrame("Frame","DeathNoteFrame")
frame:Hide()
frame:SetBackdrop(GameTooltip:GetBackdrop())
frame:SetBackdropColor(0,0,0)
frame:SetBackdropBorderColor(0,0,0,0)
frame:SetWidth(240)
frame:SetHeight(155)
frame:SetPoint("CENTER",0,0)
tinsert(UISpecialFrames,"DeathNoteFrame")

local title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
title:SetPoint("TOPLEFT", 10, -10)
title:SetText("DeathNote")

local subtitle1 = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
subtitle1:SetPoint("TOPLEFT", title, "TOPLEFT", 10, -20)
subtitle1:SetText("Who to add?")

local editbox1 = CreateFrame('EditBox', "DeathNoteEditbox1", frame)
editbox1:Show()
editbox1:SetAutoFocus(false)
editbox1:SetHeight(32)
editbox1:SetWidth(200)
editbox1:SetFontObject('GameFontHighlightSmall')
editbox1:SetPoint("TOPLEFT",subtitle1,"TOPLEFT",0,-8)

local left = editbox1:CreateTexture(nil, "BACKGROUND")
left:SetWidth(8) left:SetHeight(20)
left:SetPoint("LEFT", -5, 0)
left:SetTexture("Interface\\Common\\Common-Input-Border")
left:SetTexCoord(0, 0.0625, 0, 0.625)

local right = editbox1:CreateTexture(nil, "BACKGROUND")
right:SetWidth(8) right:SetHeight(20)
right:SetPoint("RIGHT", 0, 0)
right:SetTexture("Interface\\Common\\Common-Input-Border")
right:SetTexCoord(0.9375, 1, 0, 0.625)

local center = editbox1:CreateTexture(nil, "BACKGROUND")
center:SetHeight(20)
center:SetPoint("RIGHT", right, "LEFT", 0, 0)
center:SetPoint("LEFT", left, "RIGHT", 0, 0)
center:SetTexture("Interface\\Common\\Common-Input-Border")
center:SetTexCoord(0.0625, 0.9375, 0, 0.625)

editbox1:SetScript("OnTabPressed", function(self)
	self:ClearFocus()
	DeathNoteEditbox2:SetFocus()
end)
editbox1:SetScript("OnEnterPressed", function(self)
	self:ClearFocus()
	self:HighlightText(0,0)
	DeathNoteEditbox2:SetFocus()
	DeathNoteEditbox2:HighlightText()
end)
editbox1:SetScript("OnEscapePressed", editbox1.ClearFocus)
editbox1:SetScript("OnEditFocusLost", function(self)
	self:ClearFocus()
	self:HighlightText(0,0)
end)
editbox1:SetScript("OnEditFocusGained", function(self)
	self:HighlightText()
end)

local subtitle2 = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
subtitle2:SetPoint("TOPLEFT", editbox1, "TOPLEFT", 0, -40)
subtitle2:SetText("Reason for adding?")

local editbox2 = CreateFrame('EditBox', "DeathNoteEditbox2", frame)
editbox2:Show()
editbox2:SetAutoFocus(false)
editbox2:SetHeight(32)
editbox2:SetWidth(200)
editbox2:SetFontObject('GameFontHighlightSmall')
editbox2:SetPoint("TOPLEFT",subtitle2,"TOPLEFT",0,-8)

local left = editbox2:CreateTexture(nil, "BACKGROUND")
left:SetWidth(8) left:SetHeight(20)
left:SetPoint("LEFT", -5, 0)
left:SetTexture("Interface\\Common\\Common-Input-Border")
left:SetTexCoord(0, 0.0625, 0, 0.625)

local right = editbox2:CreateTexture(nil, "BACKGROUND")
right:SetWidth(8) right:SetHeight(20)
right:SetPoint("RIGHT", 0, 0)
right:SetTexture("Interface\\Common\\Common-Input-Border")
right:SetTexCoord(0.9375, 1, 0, 0.625)

local center = editbox2:CreateTexture(nil, "BACKGROUND")
center:SetHeight(20)
center:SetPoint("RIGHT", right, "LEFT", 0, 0)
center:SetPoint("LEFT", left, "RIGHT", 0, 0)
center:SetTexture("Interface\\Common\\Common-Input-Border")
center:SetTexCoord(0.0625, 0.9375, 0, 0.625)

editbox2:SetScript("OnTabPressed", function(self)
	self:ClearFocus()
	DeathNoteEditbox1:SetFocus()
end)
editbox2:SetScript("OnEnterPressed", function(self)
	self:ClearFocus()
	self:HighlightText(0,0)
	DeathNoteOkay:Click()
end)
editbox2:SetScript("OnEscapePressed", editbox2.ClearFocus)
editbox2:SetScript("OnEditFocusLost", function(self)
	self:ClearFocus()
	self:HighlightText(0,0)
end)
editbox2:SetScript("OnEditFocusGained", function(self)
	self:HighlightText()
end)

local button = CreateFrame("Button", "DeathNoteClose", frame, "UIPanelCloseButton")
button:SetHeight(25)
button:SetWidth(25)
button:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0)
button:SetScript("OnClick", function()
	frame:Hide()
end)

local butt = CreateFrame("Button", "DeathNoteOkay", frame)
butt:SetWidth(80) butt:SetHeight(22)
butt:SetDisabledFontObject(GameFontDisable)
butt:SetHighlightFontObject(GameFontHighlight)
butt:SetNormalFontObject(GameFontNormal)

butt:SetNormalTexture("Interface\\Buttons\\UI-Panel-Button-Up")
butt:SetPushedTexture("Interface\\Buttons\\UI-Panel-Button-Down")
butt:SetHighlightTexture("Interface\\Buttons\\UI-Panel-Button-Highlight")
butt:SetDisabledTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
butt:GetNormalTexture():SetTexCoord(0, 0.625, 0, 0.6875)
butt:GetPushedTexture():SetTexCoord(0, 0.625, 0, 0.6875)
butt:GetHighlightTexture():SetTexCoord(0, 0.625, 0, 0.6875)
butt:GetDisabledTexture():SetTexCoord(0, 0.625, 0, 0.6875)
butt:GetHighlightTexture():SetBlendMode("ADD")
butt:SetText("Okay")
butt:SetPoint("BOTTOM","DeathNoteFrame","BOTTOM",0,10)
butt:SetScript("OnClick", function()
	local okay1,okay2
	local name = DeathNoteEditbox1:GetText()
	local note = DeathNoteEditbox2:GetText()
	if name == "" then
		DeathNoteEditbox1:SetText("You need to add a name!")
		DeathNoteEditbox1:HighlightText()
	else 
		okay1 = true
	end
	if note == "" then 
		DeathNoteEditbox2:SetText("You need to add a comment!")
		DeathNoteEditbox2:HighlightText()
	else
		okay2 = true
	end
	if okay1 and okay2 then
		local nameexists = false
		for k,v in pairs(DeathNoteDB[realmName]) do
			if string.lower(v[1]) == string.lower(name) then
				v[2] = note
				nameexists = true
			end
		end
		if not nameexists then
			table.insert(DeathNoteDB[realmName],{name,note})	
		end
		print("DeathNote: Added "..name.." in the list with reason: "..note)
		frame:Hide()
	end
end)

SLASH_DeathNote1 = "/dn"
SLASH_DeathNote2 = "/deathnote"
SlashCmdList["DeathNote"] = function(slash)
	local a,b = string.split(" ",slash,2)
	if a == "wipe" then
		table.wipe(DeathNoteDB[realmName])
		print("DeathNote: Wiped database")
	elseif a == "list" then
		ShowItemRefTooltip()
	elseif a == "add" then
		if b then
			frame:Show()
			DeathNoteEditbox1:SetText(b)
			DeathNoteEditbox2:SetText("")
			DeathNoteEditbox2:SetFocus()
		else
			if not UnitExists("target") then
				frame:Show()
				DeathNoteEditbox1:SetText("")
				DeathNoteEditbox2:SetText("")
				DeathNoteEditbox1:SetFocus()
			else
				frame:Show()
				DeathNoteEditbox1:SetText(UnitName("target"))
				DeathNoteEditbox2:SetText("")
				DeathNoteEditbox2:SetFocus()
			end
		end
	elseif a == "del" then
		if not b then 
			ShowItemRefTooltip()
		else
			local removedname = DeathNoteDB[realmName][tonumber(b)][1]
			print("DeathNote: Removed "..removedname.." from the list")
			table.remove(DeathNoteDB[realmName],b)
			if ItemRefTooltip:IsShown() then ShowItemRefTooltip() end
		end
	elseif a == "autoadd" then			
		if DeathNoteDB.autoadd then DeathNoteDB.autoadd = nil else DeathNoteDB.autoadd = true end
		print("DeathNote: Auto adding gankers "..tostring(DeathNoteDB.autoadd))
	elseif a == "noteshare" then
		if DeathNoteDB.noteshare then DeathNoteDB.noteshare = nil else DeathNoteDB.noteshare = true end
		print("DeathNote: Sharing notes with party "..tostring(DeathNoteDB.noteshare))
	elseif a == "share" then
		if not b then
			ShowItemRefTooltip()
		else
			local string = DeathNoteDB[realmName][tonumber(b)][1].."|"..DeathNoteDB[realmName][tonumber(b)][2]
			if GetNumPartyMembers() > 1 then
				SendAddonMessage("DEATHNOTE",string,"PARTY")
			end
		end
	elseif not a or a == "" then
		ShowUIPanel(ItemRefTooltip)
		if not ItemRefTooltip:IsShown() then ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE") end
		ItemRefTooltip:ClearLines()
		ItemRefTooltip:AddLine("DeathNote slashcommand:")
		ItemRefTooltip:AddLine("/deathnote wipe",1,1,1)
		ItemRefTooltip:AddLine(" - Clear the database.")
		ItemRefTooltip:AddLine("/deathnote list",1,1,1)
		ItemRefTooltip:AddLine(" - Shows the list of people to kill with reasons.")
		ItemRefTooltip:AddLine("/deathnote add [name]",1,1,1)
		ItemRefTooltip:AddLine(" - Add name in the list. Name is not required.")
		ItemRefTooltip:AddLine("/deathnote del number",1,1,1)
		ItemRefTooltip:AddLine(" - Delete name from list.")
		ItemRefTooltip:AddLine("/deathnote share number",1,1,1)
		ItemRefTooltip:AddLine(" - Share note with party.")
		ItemRefTooltip:AddLine("/deathnote autoadd",1,1,1)
		ItemRefTooltip:AddLine(" - Toggle autoadding gankers.")
		ItemRefTooltip:AddLine("/deathnote noteshare",1,1,1)
		ItemRefTooltip:AddLine(" - Toggle notesharing with party.")
		ItemRefTooltip:AddLine("/deathnote notereceive",1,1,1)
		ItemRefTooltip:AddLine(" - Toggle notereceiving from party.")
		ItemRefTooltip:Show()
	end
end